local cv_speedometer = CV_RegisterVar({
	name = "reallycoolspeedometer",
	defaultvalue = "true",
	flags = CV_SHOWMODIF,
	PossibleValue = CV_TrueFalse
})

local function SafeFreeslot(...)
	for _, item in ipairs({...})
		if rawget(_G, item) == nil
			freeslot(item)
		end
	end
end

local function L_FixedDecimal(str,maxdecimal)
	if str == nil or tostring(str) == nil
		return '<invalid FixedDecimal>'
	end
	local number = tonumber(str)
	maxdecimal = ($ != nil) and $ or 3
	if tonumber(str) == 0 return '0' end
	local polarity = abs(number)/number
	local str_polarity = (polarity < 0) and '-' or ''
	local str_whole = tostring(abs(number/FRACUNIT))
	if maxdecimal == 0
		return str_polarity..str_whole
	end
	local decimal = number%FRACUNIT
	decimal = FRACUNIT + $
	decimal = FixedMul($,FRACUNIT*10^maxdecimal)
	decimal = $>>FRACBITS
	local str_decimal = string.sub(decimal,2)
	return str_polarity..str_whole..'.'..str_decimal
end

SafeFreeslot("SPR_THND")

local function drawspeedometer(v)
	if not (cv_speedometer.value) then return end
	
	local p = consoleplayer
	
	if p.accspeed == nil then return end
	if (skins[p.skin].name == "takisthefox") then return end
	
	local ypos = hudinfo[HUD_LIVES].y-20
	local maxspeed = 200*FU
	local speed = FixedDiv(p.accspeed,maxspeed)
	local normalspeed = FixedDiv(p.normalspeed,maxspeed)
	local roll
	local nroll
	local scale = FU
	local offy2 = 0
	if (speed ~= 0)
		roll = FixedAngle(180*FU-FixedMul(180*FU,speed))
	else
		roll = FixedAngle(180*FU)
	end
	if (normalspeed ~= 0)
		nroll = FixedAngle(180*FU-FixedMul(180*FU,normalspeed))
	else
		nroll = FixedAngle(180*FU)
	end
	if AngleFixed(roll) == 0
		offy2 = -4
	end
	
	for i = 0,10
		local offy = 0
		local ra = FixedAngle(180*FU-(i*18)*FU)
		if i == 10
			offy = -4
		end
		v.drawScaled((hudinfo[HUD_LIVES].x+30)*FU,
			(ypos-8+offy)*FU,
			FU/2,
			v.getSpritePatch(SPR_THND,B,0,ra),
			V_HUDTRANS|V_SNAPTOLEFT|V_SNAPTOBOTTOM|V_PERPLAYER
		)
		if i == 5
			v.drawString((hudinfo[HUD_LIVES].x+30)*FU+(30*cos(ra)),
				(ypos-8+offy)*FU-(35*sin(ra))-(4*FU),
				"100",
				V_HUDTRANS|V_SNAPTOLEFT|V_SNAPTOBOTTOM|V_PERPLAYER,
				"thin-fixed-center"
			)	
		elseif i == 10
			v.drawString((hudinfo[HUD_LIVES].x+30)*FU+(35*cos(ra)),
				(ypos-8+offy)*FU-(35*sin(ra))-(7*FU),
				"200",
				V_HUDTRANS|V_SNAPTOLEFT|V_SNAPTOBOTTOM|V_PERPLAYER,
				"thin-fixed-center"
			)	
		end
	end
	
	v.drawScaled((hudinfo[HUD_LIVES].x+30)*FU,
		(ypos-8+offy2)*FU,
		FU/2,
		v.getSpritePatch(SPR_THND,C,0,nroll),
		V_HUDTRANS|V_SNAPTOLEFT|V_SNAPTOBOTTOM|V_PERPLAYER
	)
	
	v.drawScaled((hudinfo[HUD_LIVES].x+30)*FU,
		(ypos-8+offy2)*FU,
		FU/2,
		v.getSpritePatch(SPR_THND,A,0,roll),
		V_HUDTRANS|V_SNAPTOLEFT|V_SNAPTOBOTTOM|V_PERPLAYER
	)
	
	local scorenum = "CMBCF"
	local score = L_FixedDecimal(p.accspeed,3)
	
	local prevw
	if not prevw then prevw = 0 end
	
	for i = 1,string.len(score)
		local n = string.sub(score,i,i)
		--if n == "." then n = "DOT" end
		v.drawScaled(hudinfo[HUD_LIVES].x*FU+(prevw*scale),
			(ypos)*FU-(v.cachePatch(scorenum+n).height*FixedDiv(scale-FU,2*FU)),
			FixedDiv(scale,2*FU),
			v.cachePatch(scorenum+n),
			V_HUDTRANS|V_SNAPTOLEFT|V_SNAPTOBOTTOM|V_PERPLAYER
		)
			
		prevw = $+v.cachePatch(scorenum+n).width*4/10
	end
	
end


addHook("PlayerThink",function(p)
	if not (p.realmo and p.realmo.valid) then return end
	
	p.accspeed = FixedDiv(abs(FixedHypot(p.rmomx,p.rmomy)), p.realmo.scale)
end)

addHook("HUD",drawspeedometer,"game")